/*
 * clock_config.c
 *
 *  Created on: Jun 7, 2020
 *              Oct 30, 2020 modified for ISE4071
 *      Author: Miroslav Oljaca
 */

#include "cs.h"
#include "clock_config.h"

/**** Local Function Prototypes ***********************************************/
void clock_software_trim(void);

/**** Local Functions *********************************************************/

/*!
 * @brief Initializes the system clock.
 *        Clock source is XT1 = 32,768 Hz.
 *        MCLK  = 8 MHz
 *        SMCLK = 8 MHz
 *        ACLK  = 32,768 Hz
 *
 * @return none
 */
void
hal_clock_system_init (void)
{
    do
    {
        CSCTL7 &= ~(XT1OFFG | DCOFFG);  // Clear XT1 and DCO fault flag
        SFRIFG1 &= ~OFIFG;              // Clear oscillator-fault interrupt flag
    } while (SFRIFG1 & OFIFG);          // Test oscillator fault flag

    // MSP430FR235x, MSP430FR215x Data Sheet (slasec4d):
    // 3.2.11.1 DCO Factory Trim, page 106:

    __bis_SR_register(SCG0);            // disable FLL, SCG0 = 1 in Status Register (SR).
                                        // This bit, when set, turns off the FLL+ loop control.

    CSCTL3 = (REFOLP_0       |          // REFOLP = 0b, REFO Low Power Disabled (High Power Mode)
              SELREF__XT1CLK |          // SELREF = 00b, set XT1CLK as FLL reference source
              FLLREFDIV_0               // FLLREFDIV = 000b, f_FLLREFCLK / 1
            );

    CSCTL0 = 0x0000;                    // Clear the CSCTL0 register.

    // MSP430FR235x, MSP430FR215x Data Sheet (slasec4d):
    // XT1 = 32,768 Hz, DCO Range = 8 MHz
    // Table 5-6., page 37 and Figure 5-6., page 39: DCOFTRIM = [0..7]
    //
    CSCTL1 = (DCOFTRIMEN_1 |    // DCOFTRIMEN = 1b, Enable frequency trim
              DCOFTRIM0    |    // DCOFTRIM = 011b, middle of the range
              DCOFTRIM1    |    // [0..7] is 3.
              DCORSEL_3    |    // DCORSEL = 011b, DCO range select is 8 MHz
              DISMOD_0          // DISMOD = 0b, Modulation enabled
             );

    // MSP430FR235x, MSP430FR215x Data Sheet (slasec4d):
    // 3.2.5 Digitally Controlled Oscillator (DCO), page 104:
    //
    CSCTL2 = (FLLD_0 +      // FLLD = 000b, f_DCOCLK / 1
              243           // FLLN = 243, f_DCOCLKDIV = 8 MHz
             );

    __delay_cycles(3);          // Execute NOP three times to allow time for the settings to be applied.
    __bic_SR_register(SCG0);    // enable FLL, SCG0 = 0 in Status Register (SR).
                                // This bit, when clear, turns on the FLL+ loop control.

    // Poll until FLL is locked
    //
    while ( (CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)) ||
            (CSCTL7 & DCOFFG))
    {
        //Clear DCOFFG,DCO fault flags
        CSCTL7 &= ~(DCOFFG);

        //Clear OFIFG, oscillator fault interrupt flag
        SFRIFG1 &= ~OFIFG;
    }

    // End DCO Factory Trim

    clock_software_trim();                  // Software Trim to get the best DCOFTRIM value

    CS_initClockSignal(CS_ACLK,             // set ACLK clock source signal
                       CS_XT1CLK_SELECT,    // SELA = 00b, source is XT1 = 32,768 Hz, XT1CLK
                       CS_CLOCK_DIVIDER_1   // DIVA = 0000b, ACLK = XT1 = 32,768 Hz,
                      );                    // DIVA is always bypassed if ACLK sources from XT1 in LF mode.

    CS_initClockSignal(CS_MCLK,               // set MCLK clock source signal
                       CS_DCOCLKDIV_SELECT,   // SELMS = 000b, source is 8 MHz, DCOCLKDIV
                       CS_CLOCK_DIVIDER_1     // DIVM = 000b, MCLK = 8 MHz
                       );

    CS_initClockSignal(CS_SMCLK,               // set SMCLK clock source signal
                       CS_DCOCLKDIV_SELECT,    // SELMS = 000b, source is 8 MHz, DCOCLKDIV
                       CS_CLOCK_DIVIDER_1      // DIVS = 00b, SMCLK = 8 MHz / 1 = 8 MHz
                      );

}   /* clock_cystem_init() */


/*!
 * @brief Software trim for DCO+FLL to lock target frequency
 *        MSP430FR235x, MSP430FR215x Data Sheet (slasec4d):
 *        3.2.11.2 DCO Software Trim, page 107: "I am assuming"
 *
 * @return none
 */
void
clock_software_trim (void)
{
    unsigned int oldDcoTap = 0xffff;
    unsigned int newDcoTap = 0xffff;
    unsigned int newDcoDelta = 0xffff;
    unsigned int bestDcoDelta = 0xffff;
    unsigned int csCtl0Copy = 0;
    unsigned int csCtl1Copy = 0;
    unsigned int csCtl0Read = 0;
    unsigned int csCtl1Read = 0;
    unsigned int dcoFreqTrim = 3;
    unsigned char endLoop = 0;

    do
    {
        CSCTL0 = 0x100;                         // DCO Tap = 256
        do
        {
            CSCTL7 &= ~DCOFFG;                  // Clear DCO fault flag
        }while (CSCTL7 & DCOFFG);               // Test DCO fault flag

        __delay_cycles((unsigned long)3000 * 8);// Wait FLL lock status (FLLUNLOCK) to be stable
                                                           // Suggest to wait 24 cycles of divided FLL reference clock
        while((CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)) && ((CSCTL7 & DCOFFG) == 0));

        csCtl0Read = CSCTL0;                   // Read CSCTL0
        csCtl1Read = CSCTL1;                   // Read CSCTL1

        oldDcoTap = newDcoTap;                 // Record DCOTAP value of last time
        newDcoTap = csCtl0Read & 0x01ff;       // Get DCOTAP value of this time
        dcoFreqTrim = (csCtl1Read & 0x0070)>>4;// Get DCOFTRIM value

        if(newDcoTap < 256)                    // DCOTAP < 256
        {
            newDcoDelta = 256 - newDcoTap;     // Delta value between DCPTAP and 256
            if((oldDcoTap != 0xffff) && (oldDcoTap >= 256)) // DCOTAP cross 256
                endLoop = 1;                   // Stop while loop
            else
            {
                dcoFreqTrim--;
                CSCTL1 = (csCtl1Read & (~DCOFTRIM)) | (dcoFreqTrim<<4);
            }
        }
        else                                   // DCOTAP >= 256
        {
            newDcoDelta = newDcoTap - 256;     // Delta value between DCPTAP and 256
            if(oldDcoTap < 256)                // DCOTAP cross 256
                endLoop = 1;                   // Stop while loop
            else
            {
                dcoFreqTrim++;
                CSCTL1 = (csCtl1Read & (~DCOFTRIM)) | (dcoFreqTrim<<4);
            }
        }

        if(newDcoDelta < bestDcoDelta)         // Record DCOTAP closest to 256
        {
            csCtl0Copy = csCtl0Read;
            csCtl1Copy = csCtl1Read;
            bestDcoDelta = newDcoDelta;
        }

    }while(endLoop == 0);                      // Poll until endLoop == 1

    CSCTL0 = csCtl0Copy;                       // Reload locked DCOTAP
    CSCTL1 = csCtl1Copy;                       // Reload locked DCOFTRIM
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)); // Poll until FLL is locked

}   /* clock_software_trim() */

/*** end of file ***/
