//*****************************************************************************
//   HAL Port I/O functions
//   for the ISE4075A PIR board
//
// Texas Instruments, Inc.
// Miroslav Oljaca 11/18/2020
// *****************************************************************************

#include "rom_driverlib.h"
#include "rom_map_driverlib.h"
#include "gpio.h"
#include "port_config.h"

/**** Functions **************************************************************/

void
hal_ports_init (void)
{
    /** ISE4075A PIR board ********************************************/

    /*** Port 1 **************************************************************
     *
     * P1.0 = Unused, Output Low
     * P1.1 = First gain stag output OA0O and A1
     * P1.2 = Unused, Output Low
     * P1.3 = PIR first gain stage input, OA0+ and A3
     * P1.4 = Filtered second gain stag, A4
     * P1.5 = Second gain stag output OA1O and A5
     * P1.6 = Second gain stag input OA1- and A6
     * P1.7 = Battery voltage input, A7
     *
     */

/*! \brief DO NOT CHANGE!
 *
 *  Problem occurred with high level macros, MSP430 Driver Library. Undesired
 *  change of setting. Some macros override previous settings and
 *  port configuration is not under control. Best practice, that saves a lot
 *  of time later, is NOT to use MSP430 Driver Library. This is true especially
 *  for most critical, hardware initialization.
 *
 */

    // Digital I/O Port x: Pullup or Pulldown Resistor Enable Register
    // When SAC, analog, is used it is VERY IMPORTANT that Pullup or
    // pulldown resistors are disabled
    // Reset value = 0h.
    //
    P1REN = 0;          // This disables Pullup or Pulldown Resistor on
                        // selected pins on a selected port

    // Digital I/O Port output register (0b = low, 1b = high)
    // Reset value = Undefined.
    //
    P1DIR &= ~(BIT0 |   // P1.0
               BIT2     // P1.2
              );        // This sets selected pins on a selected port as
                        // output LOW.

    // Digital I/O Port direction register (0b = input, 1b = output)
    // Reset value = 0h.
    //
    P1DIR |= (BIT0 |    // P1.0
              BIT2      // P1.2
             );         // This sets selected pins on a selected port as
                        // OUTPUT pins.

    // I/O Port function selection. General-purpose I/O is selected
    // Reset value = 0h.
    //
    P1SEL0 &= ~(BIT0 |  // P1.0
                BIT2    // P1.2
               );

    P1SEL1 &= ~(BIT0 |  // P1.0
                BIT2    // P1.2
               );       // P1SELx = 00, selected pins on a selected port are
                        // General purpose I/O.

    // Some problem occur as with analog after reset.
    // for safety first do following
    //
    P1OUT &= ~(BIT1 |   // P1.1 - OA0O and A1
               BIT3 |   // P1.3 - OA0+ and A3
               BIT4 |   // P1.4 - A4
               BIT5 |   // P1.5 - OA1O and A5
               BIT6 |   // P1.6 - OA1- and A6
               BIT7     // P1.7 - A7
              );        // Sets output as LOW

    P1DIR &= ~(BIT1 |   // P1.1 - OA0O and A1
               BIT3 |   // P1.3 - OA0+ and A3
               BIT4 |   // P1.4 - A4
               BIT5 |   // P1.5 - OA1O and A5
               BIT6 |   // P1.6 - OA1- and A6
               BIT7     // P1.7 - A7
              );        // Sets pins as INPUT

    // I/O Port function selection. Tertiary module function is selected
    // Reset value = 0h.
    //
    P1SEL0 |= (BIT1 |   // P1.1 - OA0O and A1
               BIT3 |   // P1.3 - OA0+ and A3
               BIT4 |   // P1.4 - A4
               BIT5 |   // P1.5 - OA1O and A5
               BIT6 |   // P1.6 - OA1- and A6
               BIT7     // P1.7 - A7
              );

    P1SEL1 |= (BIT1 |   // P1.1 - OA0O and A1
               BIT3 |   // P1.3 - OA0+ and A3
               BIT4 |   // P1.4 - A4
               BIT5 |   // P1.5 - OA1O and A5
               BIT6 |   // P1.6 - OA1- and A6
               BIT7     // P1.7 - A7
              );        // P1SELx = 11, selected pins on a selected port are
                        // Tertiary module function.

    /*** Port 2 **************************************************************
     *
     * P2.0 = Battery voltage output (DVCC), Output Low
     * P2.1 = GREEN LED, Output High, active Low
     * P2.4 = Unused, Output Low
     * P2.5 = Unused, Output Low
     * P2.6 = XOUT
     * P2.7 = XIN
     *
     */
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2,
                               GPIO_PIN0 |      // P2.0
                               GPIO_PIN4 |      // P2.4 - NC
                               GPIO_PIN5        // P2.5 - NC
                              );                // This function sets output LOW on the selected port's pin.

    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P2,
                                GPIO_PIN1       // P2.1 - GREEN LED
                               );               // This function sets output HIGH on the selected port's pin.

    MAP_GPIO_setAsOutputPin(GPIO_PORT_P2,
                            GPIO_PIN0 |         // P2.0
                            GPIO_PIN1 |         // P2.1 - GREEN LED
                            GPIO_PIN4 |         // P2.4 - NC
                            GPIO_PIN5           // P2.5 - NC
                           );                   // This function selected pins on a selected port as output pins.

    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P2,
                                                   GPIO_PIN6 |      // P2.6 - XOUT
                                                   GPIO_PIN7,       // P2.7 - XIN
                                                   GPIO_SECONDARY_MODULE_FUNCTION
                                                  );                // This function configures the peripheral module function in the input direction for the selected pin.

    /*** Port 3 **************************************************************
     *
     * P3.0 = Unused, Output Low
     * P3.1 = First gain stage input (dc offset), OA0-
     * P3.2 = Unused, Output Low
     * P3.3 = Unused, Output Low
     * P3.4 = Unused, Output Low
     * P3.5 = Dc offset output, OA3O
     * P3.6 = SW1, Input Pull-up
     * P3.7 = Unused, Output Low
     *
     */
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P3,
                               GPIO_PIN0 |      // P3.0 - NC
                               GPIO_PIN2 |      // P3.2 - NC
                               GPIO_PIN3 |      // P3.3 - NC
                               GPIO_PIN4 |      // P3.4 - NC
                               GPIO_PIN7        // P3.7 - NC
                              );                // This function sets output LOW on the selected port's pin.

    MAP_GPIO_setAsOutputPin(GPIO_PORT_P3,
                            GPIO_PIN0 |      // P3.0 - NC
                            GPIO_PIN2 |      // P3.2 - NC
                            GPIO_PIN3 |      // P3.3 - NC
                            GPIO_PIN4 |      // P3.4 - NC
                            GPIO_PIN7        // P3.7 - NC
                           );                // This function selected pins on a selected port as output pins.

    MAP_GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P3,
                                             GPIO_PIN6      // P3.6 - SW1
                                            );              // This function sets the selected Pin in input Mode with Pull Up resistor.

    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P3,
                                                    GPIO_PIN1 |     // P3.1 - OA0-
                                                    GPIO_PIN5,      // P3.5 - OA3O
                                                    GPIO_TERNARY_MODULE_FUNCTION
                                                   );               // This function configures the peripheral module function in the output direction for the selected pin.

    /*** Port 4 **************************************************************
     *
     * P4.0 = RED LED, Output High, active Low
     * P4.1 = DIP SW3 A, Input Pull-down
     * P4.2 = DIP SW3 B, Input Pull-down or BCL UART_RXD, Back-channel UART to the PC
     * P4.3 = DIP SW3 C, Input Pull-down or BCL UART_TXD, Back-channel UART to the PC
     * P4.6 = UCB1SDA
     * P4.7 = UCB1SCL
     *
     */
    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P4,
                                GPIO_PIN0       // P4.0 - RED LED
                               );               // This function sets output HIGH on the selected port's pin.

    MAP_GPIO_setAsOutputPin(GPIO_PORT_P4,
                            GPIO_PIN0       // P4.0 - RED LED
                           );               // This function selected pins on a selected port as output pins.

    MAP_GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P4,
                                               GPIO_PIN1        // P4.1 - DIP SW3 A
                                              );                // This function sets the selected Pin in input Mode with Pull down resistor.

    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P4,
                                                   GPIO_PIN2 |      // P4.2 UCA1RXD
                                                   GPIO_PIN3 |      // P4.3 UCA1TXD
                                                   GPIO_PIN6 |      // P4.6 UCB1SDA
                                                   GPIO_PIN7,       // P4.7 UCB1SCL
                                                   GPIO_PRIMARY_MODULE_FUNCTION     // P4SELx = 01
                                                  );                // This function configures the peripheral module function in the input direction for the selected pin.

    PM5CTL0 &= ~LOCKLPM5;       // Disable the GPIO power-on default high-impedance mode

}   /* ports_init() */

/*** end of file ***/
