/*
 * sac_config.c
 *
 *  Created on: Nov 18, 2020
 *      Author: Miro Oljaca
 */

#include "sac.h"
#include "sac_config.h"

/**** Functions **************************************************************/

//! \brief Initializes the Smart Analog Combo
//!        ADC, DAC and OA
//! \return none
void
hal_sac_init (void)
{
/*!
 * @brief SAC0OA module is configure as non-inverting op amp
 *        with variable gain [2-33]. This is first gain stage.
 * @param[in]  P1.3 OA0-, Analog signal, also ADC channel  A3
 * @param[out] P1.1 OA0O, Analog signal, also ADC channel  A1
 * @return none.
 */

    // SAC0 OA Control Register
    //
    SAC0OA = (PSEL_0   |        // PSEL = 00b, External source selected
              NSEL_1   |        // NSEL = 01b, PGA source selected
              // OAPM_1   |        // OAPM = 1b, Low speed and low power
              PMUXEN_1 |        // PMUXEN = 1b, All positive input sources are connected to OA positive port
              NMUXEN_1          // NMUXEN = 1b, All negative input sources are connected to OA negative port
             );

    // SAC0 PGA Control Register
    //
    SAC0PGA = (MSEL_3 |         // MSEL = 11b, Cascade OA Inverting mode
               (GAIN2 + GAIN0)  // GAIN = 101b, Gain select, Table 20-2 page 523
              );                // (GAIN0 + GAIN1 + GAIN2)

    // SAC0 OA Control Register
    //
    SAC0OA |= (OAEN_1  |        // OAEN = 1b, SAC OA is enabled, normal mode
               SACEN_1          // SACEN = 1b, SAC all modules are enabled, normal mode
              );

/*!
 * @brief SAC1OA module is configure as inverting op amp
 *        with variable gain [1-32] and offset control using DAC 1.
 *        This is second gain stage.
 * @param[in]  P1.6 Analog signal, also ADC channel A6
 * @param[in]  SAC1DAC
 * @param[out] P1.5 Analog signal, also ADC channel A5
 * @return none.
 */
    // SAC1 DAC Control Register
    //
    SAC1DAC = DACSREF_1;        // DACSREF = 1b, secondary reference,
                                // Internal shared reference (1.5, 2.0, or 2.5 V )
                                // DACLSEL = 00b, DAC latch loads when DACDAT written
                                // DACIE = 0b, Interrupt disabled
                                // DACEN = 0b, DAC disabled
    SAC1DAT = 2048;             // Initial output value of DAC
    SAC1DAC |= DACEN;           // SAC DAC enable

    // SAC1 OA Control Register
    //
    SAC1OA = (PSEL_1   |        // PSEL = 01b, 12-bit reference DAC source selected
              NSEL_1   |        // NSEL = 01b, PGA source selected
              // OAPM_1   |        // OAPM = 1b, Low speed and low power
              PMUXEN_1 |        // PMUXEN = 1b, All positive input sources are connected to OA positive port
              NMUXEN_1          // NMUXEN = 1b, All negative input sources are connected to OA negative port
             );

    // SAC1 PGA Control Register
    //
    SAC1PGA = (MSEL_0 |         // MSEL = 00b, Inverting PGA mode (external pad IN- is selected)
               (GAIN2 + GAIN0)  // GAIN = 101b, Gain select, Table 20-2 page 523
              );                // (GAIN0 + GAIN1 + GAIN2)

    // SAC1 OA Control Register
    //
    SAC1OA |= (OAEN_1  |        // OAEN = 1b, SAC OA is enabled, normal mode
               SACEN_1          // SACEN = 1b, SAC all modules are enabled, normal mode
              );

/*!
 * @brief SAC3OA module is configure as buffer op amp
 *        with gain of 1 and input from DAC 3.
 *        This is part of the first gain stage.
 * @param[in]  SAC3DAC
 * @param[out] P3.5 OA3O, Analog signal
 * @return none.
 */
    // SAC3 DAC Control Register
    //
    SAC3DAC = DACSREF_1;        // DACSREF = 1b, secondary reference,
                                // Internal shared reference (1.5, 2.0, or 2.5 V )
                                // DACLSEL = 00b, DAC latch loads when DACDAT written
                                // DACIE = 0b, Interrupt disabled
                                // DACEN = 0b, DAC disabled
    SAC3DAT = 2048;             // Initial output value of DAC
    SAC3DAC |= DACEN;           // SAC DAC enable

    // SAC3 OA Control Register
    //
    SAC3OA = (PSEL_1   |        // PSEL = 01b, 12-bit reference DAC source selected
              NSEL_1   |        // NSEL = 01b, PGA source selected
              // OAPM_1   |        // OAPM = 1b, Low speed and low power
              PMUXEN_1 |        // PMUXEN = 1b, All positive input sources are connected to OA positive port
              NMUXEN_1          // NMUXEN = 1b, All negative input sources are connected to OA negative port
             );

    // SAC3 PGA Control Register
    //
    SAC3PGA = MSEL_1;           // MSEL = 01b, Buffer mode (floating is selected )

    // SAC3 OA Control Register
    //
    SAC3OA |= (OAEN_1  |        // OAEN = 1b, SAC OA is enabled, normal mode
               SACEN_1          // SACEN = 1b, SAC all modules are enabled, normal mode
              );
}   /* hal_sac_init() */

/*** end of file ***/
