/*
 * timer_config.c
 *
 *  Created on: Nov 19, 2020
 *      Author: Miro Oljaca
 */
#include "timer_b.h"
#include "timer_config.h"

/**** Local Function Prototypes ***********************************************/
void hal_timer_b0_init(void);
void hal_timer_b1_init(void);
void hal_timer_b2_init(void);

/**** ISE4075A PIR board *********************************************/
//
// Clock System: XT1 = 32,768, FLLN = 243, 7,995,392 Hz = XT1 * (FLLN + 1)
//               MCLK = 8 MHz, SMCLK = 8 MHz
//
// Timer configuration:
//
// TB0.0 as 20 sps (50 ms) ISR. Using ACLK
// 32,768 / 20 = 1,638
//
// TB1.0 as 153 us delay. Using SMCLK
// 7,995,392 * 153 us = 1,223
//
#define SAMPLING_FREQUENCY_DIVIDER      1638       //for 20 sps
//#define SAMPLING_FREQUENCY_DIVIDER      16      // for 2 ksps
#define TIME_DELAY_DIVIDER              1223

/**** Functions **************************************************************/

/*!
 * @brief Initializes TB0, TB1 and TB2.
 *
 * @return none
 */
void
hal_timers_init (void)
{
    hal_timer_b0_init();
    hal_timer_b1_init();
    hal_timer_b2_init();

}   /* hal_timers_init() */


/*!
 * @brief Initializes Timer Block 0, TB0.
 *        Input clock is ACLK = 32,768 Hz
 *
 * @return none
 */
void
hal_timer_b0_init (void)
{
    //******************************************************************************
    //                  Timer0_B3 (TB0)
    //******************************************************************************
    //
    TB0CTL  &= ~MC_3;       // TB0 Stop timer

    // Clear timer
    Timer_B_clear(TB0_BASE);

    // TB0 Timer Block
    TB0CTL = (TBCLGRP_0     |   // TBCLGRP = 00b, Each TBxCLn latch loads independently
              CNTL__16      |   // CNTL = 00b, 16-bit, TBxR(max) = 0FFFFh
              TBSSEL__ACLK  |   // TBSSEL = 01b, TB0 clock source ACLK
              ID_0          |   // ID = 00b, input divider /1
              MC_1          |   // MC = 01b, Timer counts up to TBxCL0
              TBCLR             // TBCLR = 1b, TimerB clear
                                // TBIE = 0b, Interrupt disable
             );

    // Timer TB0 capture/compare register 0.
    TB0CCR0 = SAMPLING_FREQUENCY_DIVIDER - 1;       // TB0CL0 (TB0CCR0) defines the time period. The number
                                                    // of timer counts in the period is TBxCL0 + 1.

    // Timer TB0 capture/compare control register 0.
    TB0CCTL0 &= ~(CCIE +        // Clear interrupt enabled ( CCIE = 0b, interrupt disabled)
                  OUTMOD_7      // Clear output mode (OUTMOD = 000b, OUT bit value)
                 );

}   /* hal_timer_b0_init() */

/*!
 * @brief Initializes Timer Block 1, TB1.
 *        Input clock is SMCLK = 7,995,392 Hz
 *
 * @return none
 */
void
hal_timer_b1_init (void)
{
    //******************************************************************************
    //                  Timer1_B3 (TB1)
    //******************************************************************************
    //
    TB1CTL  &= ~MC_3;       // TB1 Stop timer

    // Clear timer
    Timer_B_clear(TB1_BASE);

    // TB1 Timer Block
    TB1CTL = (TBCLGRP_0     |   // TBCLGRP = 00b, Each TBxCLn latch loads independently
              CNTL__16      |   // CNTL = 00b, 16-bit, TBxR(max) = 0FFFFh
              TBSSEL__SMCLK |   // TBSSEL = 10b, TB1 clock source SMCLK
              ID_0          |   // ID = 00b, input divider /1
              MC_1          |   // MC = 01b, Timer counts up to TBxCL0
              TBCLR             // TBCLR = 1b, TimerB clear
                                // TBIE = 0b, Interrupt disable
             );

    // Timer TB1 capture/compare register 0.
    TB1CCR0 = TIME_DELAY_DIVIDER - 1;           // TB1CL0 (TB1CCR0) defines the time period. The number
                                                // of timer counts in the period is TBxCL0 + 1.

    // Timer TB1 capture/compare control register 0.
    TB1CCTL0 &= ~(CCIE +        // Clear interrupt enabled ( CCIE = 0b, interrupt disabled)
                  OUTMOD_7      // Clear output mode (OUTMOD = 000b, OUT bit value)
                 );

}   /* hal_timer_b1_init() */


/*! \brief Initializes Timer Block 2, TB2.
 *         Input clock is ACLK = 32,768 Hz, used for UI LED.
 *
 *  \return none
 */
void
hal_timer_b2_init (void)
{
#   define LED_TIME_DELAY_DIVIDER   32768
    //******************************************************************************
    //                  Timer2_B3 (TB1)
    //******************************************************************************
    //
    TB2CTL  &= ~MC_3;       // TB1 Stop timer

    // Clear timer
    Timer_B_clear(TB2_BASE);

    // TB2 Timer Block
    TB2CTL = (TBCLGRP_0     |   // TBCLGRP = 00b, Each TBxCLn latch loads independently
              CNTL__16      |   // CNTL = 00b, 16-bit, TBxR(max) = 0FFFFh
              TBSSEL__ACLK  |   // TBSSEL = 01b, TB2 clock source ACLK
              ID_0          |   // ID = 00b, input divider /1
              MC_1          |   // MC = 01b, Timer counts up to TBxCL0
              TBCLR             // TBCLR = 1b, TimerB clear
                                // TBIE = 0b, Interrupt disable
             );

    // Timer TB2 capture/compare register 0.
    TB2CCR0 = LED_TIME_DELAY_DIVIDER - 1;       // TB2CL0 (TB2CCR0) defines the time period. The number
                                                // of timer counts in the period is TBxCL0 + 1.

    // Timer TB2 capture/compare control register 0.
    TB2CCTL0 &= ~(CCIE +        // Clear interrupt enabled ( CCIE = 0b, interrupt disabled)
                  OUTMOD_7      // Clear output mode (OUTMOD = 000b, OUT bit value)
                 );

}   /* hal_timer_b2_init() */


/*!
 * @brief Starts Timer0_B3 (TB0)
 *
 * @return none
 */
void
hal_timer_b0_start (void)
{
  //  important  TBCLR clear TBxR, clock divider state, and the counter direction

    TB0CCTL0 &= ~CCIFG;         // TB0 Clear interrupt
    TB0CCTL0 |= CCIE;           // TB0 Enable interrupt
    TB0CTL |= TIMER_B_UP_MODE;  // TB0 Start timer

}   /* timer_b0_start() */


/*!
 * @brief Stops Timer0_B3 (TB0)
 *
 * @return none
 */
void
hal_timer_b0_stop (void)
{
    TB0CTL  &= ~MC_3;       // TB0 Stop timer
    TB0CCTL0 &= ~CCIE;      // TB0 Disable interrupt
    TB0CCTL0 &= ~CCIFG;     // TB0 Clear interrupt

}   /* timer0_b3_stop() */

/*!
 * @brief Starts Timer1_B3 (TB1)
 *
 * @return none
 */
void
hal_timer_b1_start (void)
{
    TB1CCTL0 &= ~CCIFG;         // TB1 Clear interrupt
    TB1CCTL0 |= CCIE;           // TB1 Enable interrupt
    TB1CTL |= TIMER_B_UP_MODE;  // TB1 Start timer

}   /* timer_b1_start() */


/*!
 * @brief Stops Timer1_B3 (TB1)
 *
 * @return none
 */
void
hal_timer_b1_stop (void)
{
    TB1CTL  &= ~MC_3;       // TB1 Stop timer
    TB1CCTL0 &= ~CCIE;      // TB1 Disable interrupt
    TB1CCTL0 &= ~CCIFG;     // TB1 Clear interrupt

}   /* timer1_b3_stop() */

/*** end of file ***/
