/*
 * motion_detection.c
 *
 *  Created on: Dec 2, 2020
 *      Author: a0873438
 */
#include "motion_detection.h"
#include <stdint.h>
#include <stdbool.h>
#include "ui_led.h"



/**** Local Function Prototypes ***********************************************/

static inline uint32_t ema_inst_val(uint32_t);
static inline uint32_t abs_inst_dev(int32_t);
static inline uint32_t inst_detn_th(uint32_t);

/**** Local Variables ********************************************************/

uint32_t reference_level = 0;
int32_t inst_dev = 0;
uint32_t abs_inst_dev_value;
uint32_t inst_detn_th_val;
// bool motion_detected = false;
uint16_t motion_detected;

unsigned int PIR_motion_count_value = 0;

/**** Functions **************************************************************/

/*!
 *
 */
void
signal_analysis (uint32_t inst_value)
{
    reference_level = ema_inst_val(inst_value);
    inst_dev = inst_value - reference_level;


    abs_inst_dev_value = abs_inst_dev(inst_dev);

    inst_detn_th_val = inst_detn_th(abs_inst_dev_value);

#ifdef DATALOGGING_ENABLE
    //values leading to Motion Detected
    PIR_Inst_Detn_th_val[PIR_Data_pointer] = inst_detn_th_val;  // Just testing the datalogging
    PIR_abs_inst_dev_value[PIR_Data_pointer] = abs_inst_dev_value;  // Just testing the datalogging
#endif



    if (abs_inst_dev_value > inst_detn_th_val)
    {
        PIR_motion_count_value = PIR_motion_count_value + (abs_inst_dev_value - inst_detn_th_val);  // Summing the area between these two curves to better filter out noise/EMI
//        led_motion_detected();
//        motion_detected = 1;
    }
    else
    {
        PIR_motion_count_value =0;
//        motion_detected = 0;
    }


    if(PIR_motion_count_value > PIR_MOTION_COUNT_THRESHOLD){
        led_motion_detected();
        motion_detected = 1;
    }
    else
    {
        motion_detected = 0;
    }

#ifdef DATALOGGING_ENABLE
    //Logging the Gained Cascade Input Voltage
    PIR_motion_detection_logging[PIR_Data_pointer] = motion_detected;  // Just testing the datalogging
#endif

//    fram_write(abs_inst_dev_value, inst_detn_th_val, motion_detected, 44 );

}   /* signal_analysis() */


/*! \brief Calculates instantaneous detection threshold value.
 *
 *  \return y(n) = |x(n)|
 */
static inline uint32_t
inst_detn_th (uint32_t detn_th_xn)
{
#   define A 2;

    static bool detn_th_init = false;   // initial value of EMA filter
    /*
     *  EMA filter coefficient alpha = 1 / 2^n, for 1/32 => n = 5
     *  Reciprocal of alpha, 1 / alpha = 2^n
     *
     *  Note: n depends on the cutoff frequency and sampling time.
     */
#   define  ALPHA_SHIFT      5      // Shift applied after accumulation, n
    /*
     * From previous conversion, replacing (n) with (n-1) accumulator has
     * following value: result = x(n-1) + (1 / alpha) * y(n-2) - y(n-2).
     * As this is recursive filter using output value from previous
     * conversion, that is same as: result = (1 / alpha) * y(n-1).
     *
     * x(n)             = ema_xn
     * y(n-1) = y(n)    = ema_yn
     * [(1/alpha)*y(n)] = ema_alpha_yn
     */
    static uint32_t detn_th_ema_yn;
    static uint32_t detn_th_ema_alpha_yn;

    if ( detn_th_init == false)   // First pass after reset, need to initialize EMA
    {
        // y(0) = x(0)
        detn_th_ema_yn = detn_th_xn;

        // [(1/alpha)*y(0)] = (1/alpha)*y(0)
        detn_th_ema_alpha_yn = detn_th_ema_yn << ALPHA_SHIFT;

        detn_th_init = true;     // EMA is initialized.
    }
    else
    {
        // [(1/alpha)*y(n)] = [(1/alpha)*y(n-1)] - y(n-1)
        detn_th_ema_alpha_yn -= detn_th_ema_yn;

        // [(1/alpha)*y(n)] = [(1/alpha)*y(n)] + x(n)
        detn_th_ema_alpha_yn += detn_th_xn;

        // y(n-1) <= y(n) = [1/alpha*y(n)] / (1/alpha)
        detn_th_ema_yn = detn_th_ema_alpha_yn >> ALPHA_SHIFT;
    }
    return (detn_th_ema_yn << 1);   // A = 2

}   /* inst_detn_th() */


/*! \brief Calculates absolute value of instantaneous deviation of the signal.
 *
 *  \return y(n) = |x(n)|
 */
static inline uint32_t
abs_inst_dev (int32_t xn)
{
    uint32_t yn;

    if (xn<0)
    {
        yn = (uint32_t) ~xn + 1;
    }
    else
    {
        yn = (uint32_t) xn;
    }

    return(yn);

}   /* abs_inst_dev() */


/*! \brief Calculates EMA on instantaneous value x(n).
 *
 *  Exponential moving average filter EMA is define as
 *  y(n) = alpha*x(n) + (1-alpha)*y(n-1)
 *
 *  \return y(n) = EMA(x(n))
 */
static inline uint32_t
ema_inst_val (uint32_t xn)
{
    static bool ema_init = false;   // initial value of EMA filter
    /*
     *  EMA filter coefficient alpha = 1 / 2^n, for 1/32 => n = 5
     *  Reciprocal of alpha, 1 / alpha = 2^n
     *
     *  Note: n depends on the cutoff frequency and sampling time.
     */
#   define  ALPHA_SHIFT      5      // Shift applied after accumulation, n
    /*
     * From previous conversion, replacing (n) with (n-1) accumulator has
     * following value: result = x(n-1) + (1 / alpha) * y(n-2) - y(n-2).
     * As this is recursive filter using output value from previous
     * conversion, that is same as: result = (1 / alpha) * y(n-1).
     *
     * x(n)             = ema_xn
     * y(n-1) = y(n)    = ema_yn
     * [(1/alpha)*y(n)] = ema_alpha_yn
     */
    static uint32_t ema_yn;
    static uint32_t ema_alpha_yn;

    if ( ema_init == false)   // First pass after reset, need to initialize EMA
    {
        // y(0) = x(0)
        ema_yn = xn;

        // [(1/alpha)*y(0)] = (1/alpha)*y(0)
        ema_alpha_yn = ema_yn << ALPHA_SHIFT;

        ema_init = true;     // EMA is initialized.
   }
   else
   {
        // [(1/alpha)*y(n)] = [(1/alpha)*y(n-1)] - y(n-1)
        ema_alpha_yn -= ema_yn;

        // [(1/alpha)*y(n)] = [(1/alpha)*y(n)] + x(n)
        ema_alpha_yn += xn;

        // y(n-1) <= y(n) = [1/alpha*y(n)] / (1/alpha)
        ema_yn = ema_alpha_yn >> ALPHA_SHIFT;
    }
    return (ema_yn);

}   /*ema_inst_val */


/*** end of file ***/
