/*
 * remove_pir_dc_voltage.c
 *
 *  Created on: Dec 1, 2020
 *      Author: a0873438
 */
#include "remove_pir_dc_voltage.h"
#include <msp430.h>
#include <stdint.h>
#include <stdbool.h>
#include "time_delay.h"




// fix header

    // For G1 = 17, G1 -1 = 16 = 2^4
#   define G1_SHIFT        4        // Shift applied for (G1-1) multiplication


/**** Local Function Prototypes ***********************************************/

static inline void enable_vref(void);
static inline void enable_sac3_sac0(void);
static inline void calc_pir_voltage(void);
static inline void ema_gained_input_voltage(void);
static inline void calc_pir_dac_dc_voltage(void);

/**** Local Variables ********************************************************/

uint32_t gained_input_voltage_value = 0;    // Calculated G1*Vin1(n)
static uint32_t ema_output_value_yn;        // y(n) <= EMA(G1*Vin1(n))

/**** Functions **************************************************************/

void
remove_pir_dc_voltage (void)
{
    enable_sac3_sac0();
    calc_pir_voltage();
    ema_gained_input_voltage();
    calc_pir_dac_dc_voltage();

}   /* remove_pir_dc_voltage() */


/*! \brief Calculate Vin1 (PIR voltage, OA0+) from Vout1 (A6, OA0O)
 *
 *  We use Vout1 and Vdac1 (SAC3 DAC) values to calculate equivalent input voltage.
 *  This is done for the fix gain of the first stage, G1 = 17.
 *      1) Measure first gain stage output voltage Vout1, A6.
 *      2) Calculate input voltage as: Vin1 = (Vout1 + (G1 - 1)*Vdac1)/G1
 *
 *  \return G1*Vin1
 */
static inline void
calc_pir_voltage (void)
{
    uint16_t v_out_value;           // A6
    uint16_t v_dac_value;           // SAC3 DAC

/* ------------------------------------------------------------------------- */
    // ADC Channel A6 (OA0O signal)
    //
    ADCCTL0 &= ~ADCENC;                 // To modify control bits must disable ADC.
    ADCMCTL0 = ADCINCH_6 | ADCSREF_1;   // Configure ADC to sample channel A6 with
                                        // V(R+) = VREF and V(R-) = AVSS
    ADCCTL0 |= ADCENC | ADCSC;          // Sampling and conversion start
    __low_power_mode_0();               // Returning here from ADC ISR in AM
/* ------------------------------------------------------------------------- */
    v_out_value = ADCMEM0;
    v_dac_value = SAC3DAT;

    /*
     * G*Vin(n) = Vout + (G-1)*Vdac
     *
     * Vout = A6
     * Vdac = SAC3 DAC
     * G = 17 => G - 1 = 16 = 2^4
     */

    // G*Vin(n) = (G - 1)*Vdac
    gained_input_voltage_value = (uint32_t) v_dac_value << G1_SHIFT;

    // G*Vin(n) = G*Vin(n) + Vout
    gained_input_voltage_value += (uint32_t) v_out_value;

}   /* calc_pir_voltage() */


/*! \brief Calculates EMA on G*Vin(n).
 *
 *  Exponential moving average filter EMA is define as
 *  y(n) = alpha*x(n) + (1-alpha)*y(n-1)
 *
 *  \return EMA(G1*Vin1(n))
 */
static inline void
ema_gained_input_voltage (void)
{
   static bool ema_init = false;   // initial value of EMA filter
   /*
    *  EMA filter coefficient alpha = 1 / 2^n, for 1/256 => n = 8
    *  Reciprocal of alpha, 1 / alpha = 2^n
    *
    *  Note: n depends on the cutoff frequency and sampling time.
    */
#   define  ALPHA_SHIFT      8      // Shift applied after accumulation, n
   /*
    * From previous conversion, replacing (n) with (n-1) accumulator has
    * following value: result = x(n-1) + (1 / alpha) * y(n-2) - y(n-2).
    * As this is recursive filter using output value from previous
    * conversion, that is same as: result = (1 / alpha) * y(n-1).
    *
    * x(n)             = gained_input_voltage_value
    * y(n-1) = y(n)    = ema_output_value_yn
    * [(1/alpha)*y(n)] = ema_alpha_output_value
    */
   static uint32_t ema_alpha_output_value = 0;

   if ( ema_init == false)   // First pass after reset, need to initialize EMA
   {
       // y(0) = x(0)
       ema_output_value_yn = gained_input_voltage_value;

       // [(1/alpha)*y(0)] = (1/alpha)*y(0)
       ema_alpha_output_value = ema_output_value_yn << ALPHA_SHIFT;

       ema_init = true;     // EMA is initialized.
   }
   else
   {
        // [(1/alpha)*y(n)] = [(1/alpha)*y(n-1)] - y(n-1)
        ema_alpha_output_value -= ema_output_value_yn;

        // [(1/alpha)*y(n)] = [(1/alpha)*y(n)] + x(n)
        ema_alpha_output_value += gained_input_voltage_value;

        // y(n-1) <= y(n) = [1/alpha*y(n)] / (1/alpha)
        ema_output_value_yn = ema_alpha_output_value >> ALPHA_SHIFT;
   }
}   /* ema_calc_pir_voltage() */


/*! \brief Calc DAC
 *
 * Vdac(n) = (G*Vin(n) - Vout(n)) / (G-1)
 *
 * G*Vin(n) = ema_output_value_yn
 * Vout = FSR/2
 *
 *  \return none
 */
static inline void
calc_pir_dac_dc_voltage (void)
{
#   define HALF_FSR     2048    // Full Scale Range is 4095, 12 bit ADC
    uint32_t pir_dac_dc_value;

    pir_dac_dc_value = ema_output_value_yn;     // EMA(G*Vin(n))
    pir_dac_dc_value -= (uint32_t) HALF_FSR;    // -Vout(n) = - FSR/2, desired

    pir_dac_dc_value >>= G1_SHIFT;      // pir_dac_dc_value / (G1 - 1)

    SAC3DAT = (uint16_t) pir_dac_dc_value;

}   /* calc_pir_dac_dc_voltage */


/*! \brief Enables SAC3 (DAC and OA3) and SAC0 (OA0).
 *
 *  1) Enable Vref. First bandgap and after voltage generator
 *  2) Enable SAC3 DAC, SAC3 OA and SAC3
 *  3) Check if SAC3 DAC value was initialized.
 *      If not:
 *          3.1 Measure PIR voltage, A3.
 *          3.2 Update SAC3DAC value with A3
 *          3.3 Wait 500 us for DAC output to settle
 *  4) Enable SAC0 OA and SAC0
 *
 *  \return none
 */
static inline void
enable_sac3_sac0 (void)
{
    static bool dac3_init = false;          // initial value of SAC3 DAC

    enable_vref ();

    SAC3DAC |= DACEN;       // Enable SAC3 DAC.
    SAC3OA |= (OAEN  |      // Enable SAC3 OA.
               SACEN        // Enable SAC3.
              );

    if(dac3_init == false)  // First pass after reset, need to initialize SAC3 DAC value
    {
        // ADC Channel A3 (PIR signal)
        //
        ADCCTL0 &= ~ADCENC;                 // To modify control bits must disable ADC.
        ADCMCTL0 = ADCINCH_3 | ADCSREF_1;   // Configure ADC to sample channel A3 with
                                            // V(R+) = VREF and V(R-) = AVSS
        ADCCTL0 |= ADCENC | ADCSC;          // Sampling and conversion start
        __low_power_mode_0();               // Returning here from ADC ISR in AM

        SAC3DAT = ADCMEM0;
        time_delay_500us();     // Wait 500 us in LPM3 for SAC3 DAC signals to settle.

        dac3_init = true;
    }

    SAC0OA |= (OAEN  |      // Enable SAC0 OA.
               SACEN        // Enable SAC0.
              );            // this one requires maybe 5us delay before ADC starts

}   /*     enable_sac3_sac0() */


/*! \brief Enables internal reference (2.0V).
 *
 *  \return none
 */
static inline void
enable_vref (void)
{
    PMMCTL2 |= REFBGEN;     // Bandgap and bandgap buffer trigger. Generation
                            // of the buffered bandgap voltage is started.
    time_delay_153us();     // Wait 153 us in LPM3 for bandgap to be ready

    PMMCTL2 |= REFGEN;      // Reference generator trigger. The generation
                            // of the variable reference voltage is started.
   while (!(PMMCTL2 & REFGENRDY));     // Wait till voltage generator is ready
   PMMCTL2 |= INTREFEN;                // Enable internal reference

}   /* enable_vref() */


/*** end of file ***/
