/*
 * time_delay.c
 *
 *  Created on: Nov 28, 2020
 *      Author: Miroslav Oljaca
 */
#include "time_delay.h"
#include "timer_b.h"

/*
 **** ISE4075A PIR board *********************************************
 *
 * Clock System: XT1 = 32,768, FLLN = 243, 7,995,392 Hz = XT1 * (FLLN + 1)
 *               MCLK = 8 MHz, SMCLK = 8 MHz
 * SMCLK is clock source for TB1. Calling time_delay(), starting timer,
 * waiting for TB1.0 ISR in LPM3 and returning takes 18.95 us. For calculation
 * this time needs to be subtracted.
 */
#   define TIME_DELAY_100US     652     // 100 us - 18.95 us = 81.05 us
                                // 81.05 us / SMCLK = 648, measured 99.42 us
                                // added 4 SMCLK, 652, measured 100 us.

#   define TIME_DELAY_153US     1078    // 153 us - 18.95 us = 134.05 us
                                // 153.05 us / SMCLK = 1072, measured 152.1 us
                                // added 6 SMCLK, 1078, measured 153 us.

#   define TIME_DELAY_500US     3847    // 500 us - 18.95 us = 481.1 us
                                // 481.1 us / SMCLK = 3847, measured ??? us
                                // added ??? SMCLK, ???, measured ??? us.

/**** Local Function Prototypes ***********************************************/

void timer_b1_start (uint16_t);
void timer_b1_stop (void);

/**** Functions **************************************************************/

/*! \brief Use TB1.0 to create 100 us time delay.
 *
 *  It set up TB1.0 for time delay, goes in LPM3, and waits ISR.
 *
 *  \return none
 */
void
time_delay_100us (void)
{
    timer_b1_start(TIME_DELAY_100US);
    __enable_interrupt();
    __low_power_mode_3();   // Waiting for TB1.0 ISR
    __disable_interrupt();  // Returning here from TB1.0 ISR in AM
    timer_b1_stop();

}   /* time_delay_100us() */


/*! \brief Use TB1.0 to create 153 us time delay.
 *
 *  It set up TB1.0 for time delay, goes in LPM3, and waits ISR.
 *
 *  \return none
 */
void
time_delay_153us (void)
{
    timer_b1_start(TIME_DELAY_153US);
    __enable_interrupt();
    __low_power_mode_3();   // Waiting for TB1.0 ISR
    __disable_interrupt();  // Returning here from TB1.0 ISR in AM
    timer_b1_stop();

}   /* time_delay_153us() */


/*! \brief Use TB1.0 to create 500 us time delay.
 *
 *  It set up TB1.0 for time delay, goes in LPM3, and waits ISR.
 *
 *  \return none
 */
void
time_delay_500us (void)
{
    timer_b1_start(TIME_DELAY_500US);
    __enable_interrupt();
    __low_power_mode_3();   // Waiting for TB1.0 ISR
    __disable_interrupt();  // Returning here from TB1.0 ISR in AM
    timer_b1_stop();

}   /* time_delay_500us() */


/*! \brief Use TB1.0 to create desired time delay.
 *
 *  \return none
 */
void
timer_b1_start (uint16_t time_delay_divider)
{
   TB1CTL  &= ~MC_3;        // TB1 Stop timer
   TB1CTL  |= TBCLR;        // Clear TBxR, clock divider state, and the counter direction.

   TB1CCTL0 &= ~(CM |       // CM = 00b, Capture mode
                 CLLD |     // CLLD = 00b, TBxCLn loads on write to TBxCCRn
                 CAP        // CAP = 0b, Capture mode
                );

   TB1CCR0 = time_delay_divider - 1;    // TB1CL0 (TB1CCR0) defines the time period. The number
                                        // of timer counts in the period is TBxCL0 + 1.

   TB1CCTL0 &= ~CCIFG;         // TB1 Clear interrupt
   TB1CCTL0 |= CCIE;           // TB1 Enable interrupt
   TB1CTL |= TIMER_B_UP_MODE;  // TB1 Start timer

}   /* timer_b1_start() */


/*! \brief Stops Timer1_B3 (TB1)
 *
 *  \return none
 */
void
timer_b1_stop (void)
{
    TB1CTL  &= ~MC_3;       // TB1 Stop timer
    TB1CCTL0 &= ~CCIE;      // TB1 Disable interrupt
    TB1CCTL0 &= ~CCIFG;     // TB1 Clear interrupt

}   /* timer_b1_stop() */

/*** end of file ***/
